% M. Srinivasan, October 7, 2010
% this program is to demonstrate some simple rotation matrices

clear all; close all; clc;

% Let us first draw some axes, X_A, Y_A, Z_A corresponding to reference
% frame {A}. Say these are just [1 0 0], [0 1 0], and [0 0 1].

Aorigin = [0; 0; 0];
X_A = [1; 0; 0];
Y_A = [0; 1; 0];
Z_A = [0; 0; 1];

%%
figure(1);
plot3([Aorigin(1) Aorigin(1)+X_A(1)],[Aorigin(2) Aorigin(2)+X_A(2)],[Aorigin(3) Aorigin(3)+X_A(3)]); 
hold on;
plot3([Aorigin(1) Aorigin(1)+Y_A(1)],[Aorigin(2) Aorigin(2)+Y_A(2)],[Aorigin(3) Aorigin(3)+Y_A(3)]); 
plot3([Aorigin(1) Aorigin(1)+Z_A(1)],[Aorigin(2) Aorigin(2)+Z_A(2)],[Aorigin(3) Aorigin(3)+Z_A(3)]); 
xlabel('X_A'); ylabel('Y_A'); zlabel('Z_A');
text(Aorigin(1),Aorigin(2),Aorigin(3),'Aorigin');
% 

pause(2)

grid on;

%% New reference frame {B} is obtained by a simple translation of {A}
Borigin_A = [1; 1; 1]; % representation of Borigin in {A} namely A_P_Borigin
X_B = X_A;
Y_B = Y_A;
Z_B = Z_A;

figure(1);
plot3([Borigin_A(1) Borigin_A(1)+X_B(1)],[Borigin_A(2) Borigin_A(2)+X_B(2)],[Borigin_A(3) Borigin_A(3)+X_B(3)],'r'); 
hold on;
plot3([Borigin_A(1) Borigin_A(1)+Y_B(1)],[Borigin_A(2) Borigin_A(2)+Y_B(2)],[Borigin_A(3) Borigin_A(3)+Y_B(3)],'r'); 
plot3([Borigin_A(1) Borigin_A(1)+Z_B(1)],[Borigin_A(2) Borigin_A(2)+Z_B(2)],[Borigin_A(3) Borigin_A(3)+Z_B(3)],'r'); 
text(Borigin_A(1),Borigin_A(2),Borigin_A(3),'Borigin');

pause(2)

% 
% 
% 
%% New reference frame {C} is obtained by a simple rotation of {A}
Corigin_A = [0; 0; 0];
theta = pi/3;
Rz = [cos(theta) sin(theta) 0; -sin(theta) cos(theta) 0; 0 0 1];
X_C = Rz(1,1)*X_A + Rz(1,2)*Y_A + Rz(1,3)*Z_A;
Y_C = Rz(2,1)*X_A + Rz(2,2)*Y_A + Rz(2,3)*Z_A;
Z_C = Rz(3,1)*X_A + Rz(3,2)*Y_A + Rz(3,3)*Z_A;
% why are we not using simple matrix multiplication R*[X_A; Y_A; Z_A]?
% because each of X_A, Y_A, etc, is a 3x1 vector and R is 3x3

figure(1);
plot3([Corigin_A(1) Corigin_A(1)+X_C(1)],[Corigin_A(2) Corigin_A(2)+X_C(2)],[Corigin_A(3) Corigin_A(3)+X_C(3)],'r'); 
hold on;
plot3([Corigin_A(1) Corigin_A(1)+Y_C(1)],[Corigin_A(2) Corigin_A(2)+Y_C(2)],[Corigin_A(3) Corigin_A(3)+Y_C(3)],'r'); 
plot3([Corigin_A(1) Corigin_A(1)+Z_C(1)],[Corigin_A(2) Corigin_A(2)+Z_C(2)],[Corigin_A(3) Corigin_A(3)+Z_C(3)],'r'); 
text(Corigin_A(1),Corigin_A(2),Corigin_A(3),'Corigin');
% grid on;axis equal;
pause(2);
% 
%% a sequence of simple rotations
thetalist = linspace(-pi/3,pi,100)';
alphalist = linspace(0,pi,100)';
gammalist = linspace(0,pi,100)';
for i = 1:length(thetalist)
    
    % Z rotation
    Rz = [cos(thetalist(i)) sin(thetalist(i)) 0; -sin(thetalist(i)) cos(thetalist(i)) 0; 0 0 1];
    
    % X rotation
    Rx = [1 0 0; 0 cos(alphalist(i)) sin(alphalist(i)); 0 -sin(alphalist(i)) cos(alphalist(i))];
    
    % Y rotation
    Ry = [cos(gammalist(i)) 0 -sin(gammalist(i)); 0 1 0; sin(gammalist(i)) 0 cos(gammalist(i))];
    
    
    R = Rx*Rz*Ry;
    
%     determinantOfR = det(R); % should be 1
%     RtimesRtranspose = R*R' % should be identity
    
    X_C = R(1,1)*X_A + R(1,2)*Y_A + R(1,3)*Z_A;
    Y_C = R(2,1)*X_A + R(2,2)*Y_A + R(2,3)*Z_A;
    Z_C = R(3,1)*X_A + R(3,2)*Y_A + R(3,3)*Z_A;
    
    figure(3);
    plot3([Corigin_A(1) Corigin_A(1)+X_C(1)],[Corigin_A(2) Corigin_A(2)+X_C(2)],[Corigin_A(3) Corigin_A(3)+X_C(3)],'r'); 
    hold on;
    plot3([Corigin_A(1) Corigin_A(1)+Y_C(1)],[Corigin_A(2) Corigin_A(2)+Y_C(2)],[Corigin_A(3) Corigin_A(3)+Y_C(3)],'r'); 
    plot3([Corigin_A(1) Corigin_A(1)+Z_C(1)],[Corigin_A(2) Corigin_A(2)+Z_C(2)],[Corigin_A(3) Corigin_A(3)+Z_C(3)],'r'); 
    text(Corigin_A(1),Corigin_A(2),Corigin_A(3),'Corigin');
    grid on; axis equal;
    axis([-1 1 -1 1 -1 1]); 
    pause(.05); hold off;
end

% 
% 
% 
% 
