% M. Srinivasan, October 14, 2010
% this program is to demonstrate some rotation and translation operators

clear all; close all; clc;

%% let me draw the {A} reference frame
Aorigin = [0; 0; 0];
X_A = [1; 0; 0];
Y_A = [0; 1; 0];
Z_A = [0; 0; 1];
figure(1);
plot3([Aorigin(1) Aorigin(1)+X_A(1)],[Aorigin(2) Aorigin(2)+X_A(2)],[Aorigin(3) Aorigin(3)+X_A(3)],'r'); 
hold on;
plot3([Aorigin(1) Aorigin(1)+Y_A(1)],[Aorigin(2) Aorigin(2)+Y_A(2)],[Aorigin(3) Aorigin(3)+Y_A(3)],'r'); 
plot3([Aorigin(1) Aorigin(1)+Z_A(1)],[Aorigin(2) Aorigin(2)+Z_A(2)],[Aorigin(3) Aorigin(3)+Z_A(3)],'r'); 
xlabel('X_A'); ylabel('Y_A'); zlabel('Z_A');
text(Aorigin(1),Aorigin(2),Aorigin(3),'Aorigin');
grid on;
pause;


%% 
% % let me draw a little simplex, a tetrahedron that we can move around
Plist0 = [0 0 0; 0.5 0 0; 0 0.5 0; 0 0 0.5]; % list of points defining the tetrahedron
% note the transpose at the end, so that each point is a column. Plist0 is
% a 3 x N matrix where N is the number of points
Plist = Plist0';
figure(1);  % below I draw all the lines forming the tetrahedron
h1 = line([Plist(1,1) Plist(1,2)],[Plist(2,1) Plist(2,2)],[Plist(3,1) Plist(3,2)]);
h2 = line([Plist(1,2) Plist(1,3)],[Plist(2,2) Plist(2,3)],[Plist(3,2) Plist(3,3)]);
h3 = line([Plist(1,3) Plist(1,4)],[Plist(2,3) Plist(2,4)],[Plist(3,3) Plist(3,4)]);
h4 = line([Plist(1,4) Plist(1,1)],[Plist(2,4) Plist(2,1)],[Plist(3,4) Plist(3,1)]);
h5 = line([Plist(1,3) Plist(1,1)],[Plist(2,3) Plist(2,1)],[Plist(3,3) Plist(3,1)]);
h6 = line([Plist(1,4) Plist(1,2)],[Plist(2,4) Plist(2,2)],[Plist(3,4) Plist(3,2)]);

pause
 
%% pure translation of the little tetrahedron
Plist = Plist0 + 0.5; % adding 0.5 to every coordinate
Plist = Plist';
set(h1,'xdata',[Plist(1,1) Plist(1,2)],'ydata',[Plist(2,1) Plist(2,2)],'zdata',[Plist(3,1) Plist(3,2)]);
set(h2,'xdata',[Plist(1,2) Plist(1,3)],'ydata',[Plist(2,2) Plist(2,3)],'zdata',[Plist(3,2) Plist(3,3)]);
set(h3,'xdata',[Plist(1,3) Plist(1,4)],'ydata',[Plist(2,3) Plist(2,4)],'zdata',[Plist(3,3) Plist(3,4)]);
set(h4,'xdata',[Plist(1,4) Plist(1,1)],'ydata',[Plist(2,4) Plist(2,1)],'zdata',[Plist(3,4) Plist(3,1)]);
set(h5,'xdata',[Plist(1,3) Plist(1,1)],'ydata',[Plist(2,3) Plist(2,1)],'zdata',[Plist(3,3) Plist(3,1)]);
set(h6,'xdata',[Plist(1,4) Plist(1,2)],'ydata',[Plist(2,4) Plist(2,2)],'zdata',[Plist(3,4) Plist(3,2)]);
 
pause

%% smooth translation by small increments of x, y, and z
% Nlist = 100; 
% xmovelist = 0*linspace(0,1,100)';
% ymovelist = 0*linspace(0,1,100)';
% zmovelist = 0.5*sin(5*linspace(0,1,100)');
% for i = 1:Nlist
%         Plist = Plist0';
% 
%     Plist(1,:) = Plist0(1,:) + xmovelist(i); % changing all the x-coordinates equally
%     Plist(2,:) = Plist0(2,:) + ymovelist(i); % changing all the y-coordinates equally
%     Plist(3,:) = Plist0(3,:) + zmovelist(i); % changing all the z-coordinates equally
%     set(h1,'xdata',[Plist(1,1) Plist(1,2)],'ydata',[Plist(2,1) Plist(2,2)],'zdata',[Plist(3,1) Plist(3,2)]);
%     set(h2,'xdata',[Plist(1,2) Plist(1,3)],'ydata',[Plist(2,2) Plist(2,3)],'zdata',[Plist(3,2) Plist(3,3)]);
%     set(h3,'xdata',[Plist(1,3) Plist(1,4)],'ydata',[Plist(2,3) Plist(2,4)],'zdata',[Plist(3,3) Plist(3,4)]);
%     set(h4,'xdata',[Plist(1,4) Plist(1,1)],'ydata',[Plist(2,4) Plist(2,1)],'zdata',[Plist(3,4) Plist(3,1)]);
%     set(h5,'xdata',[Plist(1,3) Plist(1,1)],'ydata',[Plist(2,3) Plist(2,1)],'zdata',[Plist(3,3) Plist(3,1)]);
%     set(h6,'xdata',[Plist(1,4) Plist(1,2)],'ydata',[Plist(2,4) Plist(2,2)],'zdata',[Plist(3,4) Plist(3,2)]);
%     axis([-1 1 -1 1 -1 1]); hold on; % axis equal;
%     pause(0.01);
% end
% 
% pause

%% pure rotation of the whole tetrahedron (by default, the rotation is about
% the origin of the reference configuration)
Nlist = 100; 
thetalist = pi*linspace(0,2.5,Nlist)';
alphalist = pi*linspace(0,2,Nlist)';

for i = 1:Nlist
    
	% simple rotation about z
    Rz = [cos(thetalist(i)) -sin(thetalist(i)) 0; ...
        sin(thetalist(i)) cos(thetalist(i)) 0; ...
        0 0 1];
    % simple rotation about x
   Rx = [1 0 0;
       0 cos(alphalist(i)) -sin(alphalist(i)); ...
       0 sin(alphalist(i)) cos(alphalist(i)); ...
       ];
    
	% simple rotation about y
    Ry = [cos(thetalist(i)) 0 sin(thetalist(i)); ...
            0 1 0
        -sin(thetalist(i)) 0 cos(thetalist(i))];
    
    Plist = (Rx*Rz*Ry*Plist0');
%     Plist = Plist';
    
    set(h1,'xdata',[Plist(1,1) Plist(1,2)],'ydata',[Plist(2,1) Plist(2,2)],'zdata',[Plist(3,1) Plist(3,2)]);
    set(h2,'xdata',[Plist(1,2) Plist(1,3)],'ydata',[Plist(2,2) Plist(2,3)],'zdata',[Plist(3,2) Plist(3,3)]);
    set(h3,'xdata',[Plist(1,3) Plist(1,4)],'ydata',[Plist(2,3) Plist(2,4)],'zdata',[Plist(3,3) Plist(3,4)]);
    set(h4,'xdata',[Plist(1,4) Plist(1,1)],'ydata',[Plist(2,4) Plist(2,1)],'zdata',[Plist(3,4) Plist(3,1)]);
    set(h5,'xdata',[Plist(1,3) Plist(1,1)],'ydata',[Plist(2,3) Plist(2,1)],'zdata',[Plist(3,3) Plist(3,1)]);
    set(h6,'xdata',[Plist(1,4) Plist(1,2)],'ydata',[Plist(2,4) Plist(2,2)],'zdata',[Plist(3,4) Plist(3,2)]);
    axis([-1 1 -1 1 -1 1]); hold on; % axis equal;
    pause(0.01);
    
end
